@@ -2,6 +2,7 @@ import base64
 import gc
 import json
 import os
+import random
 import tempfile
 from dataclasses import dataclass
 from io import BytesIO
@@ -15,6 +16,7 @@ from diffusers import (
     AltDiffusionPipeline,
     DiffusionPipeline,
     StableDiffusionImg2ImgPipeline,
+    StableDiffusionInstructPix2PixPipeline,
     StableDiffusionPipeline,
 )
 from loguru import logger
@@ -74,9 +76,10 @@ class X2Image:
             use_auth_token=utils.use_auth_token(),
         )
         components = self.text2img_pipeline.components
-
+        self.pix2pix_pipeline = None
         if isinstance(self.text2img_pipeline, StableDiffusionPipeline):
             self.img2img_pipeline = StableDiffusionImg2ImgPipeline(**components)
+            self.pix2pix_pipeline = StableDiffusionInstructPix2PixPipeline(**components)
         elif isinstance(self.text2img_pipeline, AltDiffusionPipeline):
             self.img2img_pipeline = AltDiffusionImg2ImgPipeline(**components)
         else:
@@ -87,6 +90,8 @@ class X2Image:
         self.text2img_pipeline.safety_checker = utils.no_safety_checker
         self.img2img_pipeline.to(self.device)
         self.img2img_pipeline.safety_checker = utils.no_safety_checker
+        self.pix2pix_pipeline.to(self.device)
+        self.pix2pix_pipeline.safety_checker = utils.no_safety_checker
 
         self.compatible_schedulers = {
             scheduler.__name__: scheduler for scheduler in self.text2img_pipeline.scheduler.compatibles
@@ -121,11 +126,17 @@ class X2Image:
                 num_inference_steps=2,
             )
 
+            self.pix2pix_pipeline.enable_attention_slicing()
+            prompt = "turn him into a cyborg"
+            _ = self.pix2pix_pipeline(prompt, image=init_image, num_inference_steps=2)
+
     def _set_scheduler(self, pipeline_name, scheduler_name):
         if pipeline_name == "text2img":
             scheduler_config = self.text2img_pipeline.scheduler.config
         elif pipeline_name == "img2img":
             scheduler_config = self.img2img_pipeline.scheduler.config
+        elif pipeline_name == "pix2pix":
+            scheduler_config = self.pix2pix_pipeline.scheduler.config
         else:
             raise ValueError(f"Pipeline {pipeline_name} not supported")
 
@@ -163,6 +174,11 @@ class X2Image:
     def text2img_generate(
         self, prompt, negative_prompt, scheduler, image_size, num_images, guidance_scale, steps, seed
     ):
+
+        if seed == -1:
+            # generate random seed
+            seed = random.randint(0, 999999)
+
         generator, num_images = self._pregen(
             pipeline_name="text2img",
             scheduler=scheduler,
@@ -200,6 +216,11 @@ class X2Image:
     def img2img_generate(
         self, prompt, image, strength, negative_prompt, scheduler, num_images, guidance_scale, steps, seed
     ):
+
+        if seed == -1:
+            # generate random seed
+            seed = random.randint(0, 999999)
+
         generator, num_images = self._pregen(
             pipeline_name="img2img",
             scheduler=scheduler,
@@ -232,6 +253,46 @@ class X2Image:
         )
         return output_images, _metadata
 
+    def pix2pix_generate(
+        self, prompt, image, negative_prompt, scheduler, num_images, guidance_scale, image_guidance_scale, steps, seed
+    ):
+        if seed == -1:
+            # generate random seed
+            seed = random.randint(0, 999999)
+
+        generator, num_images = self._pregen(
+            pipeline_name="pix2pix",
+            scheduler=scheduler,
+            num_images=num_images,
+            seed=seed,
+        )
+        output_images = self.pix2pix_pipeline(
+            prompt=prompt,
+            image=image,
+            negative_prompt=negative_prompt,
+            num_inference_steps=steps,
+            guidance_scale=guidance_scale,
+            image_guidance_scale=image_guidance_scale,
+            num_images_per_prompt=num_images,
+            generator=generator,
+        ).images
+        metadata = {
+            "prompt": prompt,
+            "negative_prompt": negative_prompt,
+            "scheduler": scheduler,
+            "num_images": num_images,
+            "guidance_scale": guidance_scale,
+            "image_guidance_scale": image_guidance_scale,
+            "steps": steps,
+            "seed": seed,
+        }
+        output_images, _metadata = self._postgen(
+            metadata=metadata,
+            output_images=output_images,
+            pipeline_name="pix2pix",
+        )
+        return output_images, _metadata
+
     def app(self):
         available_schedulers = list(self.compatible_schedulers.keys())
         if "EulerAncestralDiscreteScheduler" in available_schedulers:
@@ -240,10 +301,18 @@ class X2Image:
             )
         # col3, col4 = st.columns(2)
         # with col3:
-        input_image = st.file_uploader("Upload an image to use image2image", type=["png", "jpg", "jpeg"])
+        input_image = st.file_uploader(
+            "Upload an image to use image2image or pix2pix",
+            type=["png", "jpg", "jpeg"],
+            help="Upload an image to use image2image. If left blank, text2image will be used instead.",
+        )
+        use_pix2pix = st.checkbox("Use pix2pix", value=False)
         if input_image is not None:
             input_image = Image.open(input_image)
-            pipeline_name = "img2img"
+            if use_pix2pix:
+                pipeline_name = "pix2pix"
+            else:
+                pipeline_name = "img2img"
             # display image using html
             # convert image to base64
             # st.markdown(f"<img src='data:image/png;base64,{input_image}' width='200'>", unsafe_allow_html=True)
@@ -268,22 +337,79 @@ class X2Image:
         # with col4:
         col1, col2 = st.columns(2)
         with col1:
-            prompt = st.text_area("Prompt", "Blue elephant")
+            prompt = st.text_area("Prompt", "Blue elephant", help="Prompt to guide image generation")
         with col2:
-            negative_prompt = st.text_area("Negative Prompt", "")
+            negative_prompt = st.text_area(
+                "Negative Prompt",
+                "",
+                help="The prompt not to guide image generation. Write things that you dont want to see in the image.",
+            )
         # sidebar options
-        scheduler = st.sidebar.selectbox("Scheduler", available_schedulers, index=0)
         if input_image is None:
-            image_height = st.sidebar.slider("Image height", 128, 1024, 512, 128)
-            image_width = st.sidebar.slider("Image width", 128, 1024, 512, 128)
-        guidance_scale = st.sidebar.slider("Guidance scale", 1.0, 40.0, 7.5, 0.5)
-        if input_image is not None:
-            strength = st.sidebar.slider("Denoising strength", 0.0, 1.0, 0.8, 0.05)
+            image_height = st.sidebar.slider(
+                "Image height", 128, 1024, 512, 128, help="The height in pixels of the generated image."
+            )
+            image_width = st.sidebar.slider(
+                "Image width", 128, 1024, 512, 128, help="The width in pixels of the generated image."
+            )
 
-        num_images = st.sidebar.slider("Number of images per prompt", 1, 30, 1, 1)
-        steps = st.sidebar.slider("Steps", 1, 150, 50, 1)
+        num_images = st.sidebar.slider(
+            "Number of images per prompt",
+            1,
+            30,
+            1,
+            1,
+            help="Number of images you want to generate. More images requires more time and uses more GPU memory.",
+        )
+
+        # add section advanced options
+        st.sidebar.markdown("### Advanced options")
+        scheduler = st.sidebar.selectbox(
+            "Scheduler", available_schedulers, index=0, help="Scheduler to use for generation"
+        )
+        guidance_scale = st.sidebar.slider(
+            "Guidance scale",
+            1.0,
+            40.0,
+            7.5,
+            0.5,
+            help="Higher guidance scale encourages to generate images that are closely linked to the text `prompt`, usually at the expense of lower image quality.",
+        )
+        if use_pix2pix and input_image is not None:
+            image_guidance_scale = st.sidebar.slider(
+                "Image guidance scale",
+                1.0,
+                40.0,
+                1.5,
+                0.5,
+                help="Image guidance scale is to push the generated image towards the inital image `image`. Image guidance scale is enabled by setting `image_guidance_scale > 1`. Higher image guidance scale encourages to generate images that are closely linked to the source image `image`, usually at the expense of lower image quality.",
+            )
+        if input_image is not None and not use_pix2pix:
+            strength = st.sidebar.slider(
+                "Denoising strength",
+                0.0,
+                1.0,
+                0.8,
+                0.05,
+                help="Conceptually, indicates how much to transform the reference `image`. Must be between 0 and 1. `image` will be used as a starting point, adding more noise to it the larger the `strength`. The number of denoising steps depends on the amount of noise initially added. When `strength` is 1, added noise will be maximum and the denoising process will run for the full number of iterations specified in `num_inference_steps`. A value of 1, therefore, essentially ignores `image`.",
+            )
+        steps = st.sidebar.slider(
+            "Steps",
+            1,
+            150,
+            50,
+            1,
+            help="The number of denoising steps. More denoising steps usually lead to a higher quality image at the expense of slower inference.",
+        )
+        seed = st.sidebar.number_input(
+            "Seed",
+            value=42,
+            min_value=-1,
+            max_value=999999,
+            step=1,
+            help="Random seed. Change for different results using same parameters.",
+        )
 
-        seed = st.sidebar.number_input("Seed", value=42, min_value=1, max_value=999999, step=1)
         sub_col, download_col = st.columns(2)
         with sub_col:
             submit = st.button("Generate")
@@ -313,4 +439,16 @@ class X2Image:
                         steps=steps,
                         seed=seed,
                     )
+                elif pipeline_name == "pix2pix":
+                    output_images, metadata = self.pix2pix_generate(
+                        prompt=prompt,
+                        image=input_image,
+                        negative_prompt=negative_prompt,
+                        scheduler=scheduler,
+                        num_images=num_images,
+                        guidance_scale=guidance_scale,
+                        image_guidance_scale=image_guidance_scale,
+                        steps=steps,
+                        seed=seed,
+                    )
             utils.display_and_download_images(output_images, metadata, download_col)
